##############################################################
# an extended Frame that makes menu/tool-bar's automatically
##############################################################


from Tkinter import *             # widget classes
from types   import *             # type constants

class GuiMaker(Frame):
    def __init__(self, master=None):
        Frame.__init__(self, master) 
        self.pack(expand=YES, fill=BOTH)       # make frame stretchable
        self.start()                           # subclass: set menu/toolBar
        self.makeMenuBar()                     # here: build menu-bar
        self.makeWidgets()                     # subclass: add middle-part
        self.makeToolBar()                     # here: build tool-bar

    ######################################
    # make expandable menu bar at the top
    ######################################

    menuBar    = []     # class defaults
    toolBar    = []     # change in subclasses
    helpButton = 1      # in 'start' if need 'self'

    def makeMenuBar(self):
        menubar = Frame(self, relief=RAISED, bd=2)
        menubar.pack(side=TOP, expand=YES, fill=X)

        for entry in self.menuBar:
            mbutton = Menubutton(menubar, text=entry[0], underline=entry[1])
            mbutton.pack(side=LEFT)
            menu = self.addMenuItems(mbutton, entry[2])
            mbutton['menu'] = menu

        if self.helpButton: 
            Button(menubar, text    = 'Help', 
                            cursor  = 'gumby', 
                            relief  = FLAT, 
                            command = self.help).pack(side=RIGHT)

    def addMenuItems(self, parent, items):
        menu = Menu(parent)
        for item in items:                     # scan nested items list
            if item == 'separator':            # string: add separator
                menu.add_separator({})
            elif type(item) == ListType:       # list: disabled item list
                for num in item:
                    menu.entryconfig(num, state=DISABLED)
            elif type(item[2]) != ListType:
                menu.add_command(label     = item[0],       # command: 
                                 underline = item[1],       # add command
                                 command   = item[2])       # cmd=callable
            else:
                submenu = self.addMenuItems(menu, item[2])    # sublist:
                menu.add_cascade(label     = item[0],         # make submenu
                                 underline = item[1],         # add cascad
                                 menu      = submenu) 
        return menu 

    ######################################
    # make expandable button bar at bottom
    ###################################### 

    def makeToolBar(self):
        toolbar = Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
        toolbar.pack(side=BOTTOM, expand=YES, fill=X)
        for item in self.toolBar: 
            Button(toolbar, text=item[0], command=item[1]).pack(item[2])
 
    ######################################
    # default 'middle' part: override me
    ######################################

    def makeWidgets(self):
        name = Label(self,   
                     text   = self.__class__.__name__, 
                     cursor = 'crosshair')
        name.pack(padx=50, pady=50, expand=YES, fill=BOTH)


###########################################################
# self test when file run stand-alone: 'python guimaker.py'
###########################################################

if __name__ == '__main__':
    from guimixin import *                 # GuiMixin: help, quit
    class TestApp(GuiMixin, GuiMaker):
        #helpButton = 0
        def start(self):
            self.menuBar = [ 
                ('File', 0,  
                    [('Quit',  0, self.quit)]
                )]
            self.toolBar = [
                ('Quit', self.quit, {'side': LEFT})
                ]
    TestApp().mainloop()
