##############################################################
# uses menu classes for layouts, instead of type tests
##############################################################

from Tkinter import *             # widget classes

######################
# menu layout classes 
######################

def addMenuItems(parent, items):
    menu = Menu(parent)
    for item in items:
        item.action(menu)
    return menu

class MenuCascade:
    def __init__(self, label, underline, cascade):
        self.label     = label
        self.underline = underline
        self.cascade   = cascade
    def action(self, menu):
        submenu = addMenuItems(menu, self.cascade) 
        menu.add_cascade(label     = self.label,      
                         underline = self.underline,   
                         menu      = submenu) 

class MenuItem:
    def __init__(self, label, underline, command):
        self.label     = label
        self.underline = underline
        self.command   = command
    def action(self, menu):
        menu.add_command(label     = self.label,
                         underline = self.underline,
                         command   = self.command)

class MenuSeparator:
    def action(self, menu):
        menu.add_separator({})

class MenuDisabler:
    def __init__(self, indexList):
        self.indexes = indexList
    def action(self, menu):
        for num in self.indexes:
            menu.entryconfig(num, state=DISABLED)

#####################
# new guimaker class
#####################

class GuiMaker(Frame):
    def __init__(self, master=None):
        Frame.__init__(self, master) 
        self.pack(expand=YES, fill=BOTH)         # make frame stretchable
        self.start()                             # subclass: set menu/toolBar
        self.makeMenuBar()                       # here: build menu-bar
        self.makeWidgets()                       # subclass?: add middle-part
        self.makeToolBar()                       # here: build tool-bar

    menuBar    = []   # Menu list (class data)
    toolBar    = []   # default button list
    helpButton = 1    # change in subclasses

    def makeMenuBar(self):
        menubar = Frame(self, relief=RAISED, bd=2)
        menubar.pack(side=TOP, expand=YES, fill=X)
        for entry in self.menuBar:
            mbutton = Menubutton(menubar, text=entry[0], underline=entry[1])
            mbutton.pack(side=LEFT)
            menu = addMenuItems(mbutton, entry[2])
            mbutton['menu'] = menu
        if self.helpButton: 
            Button(menubar, text    = 'Help', 
                            cursor  = 'gumby', 
                            relief  = FLAT, 
                            command = self.help).pack(side=RIGHT)

    def makeToolBar(self):
        toolbar = Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
        toolbar.pack(side=BOTTOM, expand=YES, fill=X)
        for item in self.toolBar: 
            Button(toolbar, text=item[0], command=item[1]).pack(item[2])
 
    def makeWidgets(self):
        name = Label(self,   
                     text   = self.__class__.__name__, 
                     cursor = 'crosshair')
        name.pack(padx=50, pady=50, expand=YES, fill=BOTH)


###########################
# new big_gui2 layout code
###########################

from guimixin import GuiMixin     # mix-in methods

class Hello(GuiMixin, GuiMaker):  # use new GuiMaker
    def start(self):
        self.hellos = 1
        self.master.title("GuiMaker Demo")
        self.master.iconname("GuiMaker")

        self.menuBar = [                            
          ('File', 0,                                
              [MenuItem('New...',  0, self.notdone),   
               MenuItem('Open...', 0, self.fileOpen),
               MenuItem('Quit',    0, self.quit)]   
          ),

          ('Edit', 0,
              [MenuItem('Cut',  -1, lambda:0),       
               MenuItem('Paste',-1, lambda:0),
               MenuSeparator(),              
               MenuCascade('Stuff', -1,
                   [MenuItem('Clone', -1, self.clone),
                    MenuItem('More',  -1, self.more)]),
               MenuItem('Delete', -1, lambda:0),
               MenuDisabler([5])]                     
          ),

          ('Play', 0,
              [MenuItem('Hello',     0, self.greeting),
               MenuItem('Popup...',  0, self.dialog),
               MenuCascade('Demos', 0,
                  [MenuItem('Hanoi', -1, 
                             lambda x=self: x.spawn('guido/hanoi.py', 1)),
                   MenuItem('Pong',  -1, 
                             lambda x=self: x.spawn('matt/pong-demo-1.py'))
                  ])
               ]
          )]

# [rest is the same as big_gui2.py]
