################################################################################
# App subclasses that map input/output to internal strings
################################################################################

import sys, string
from app import App

################################################################################
# an App that redirects input/output streams to internal files (strings)
################################################################################

class InternalApp(App):
    def __init__(self, text=''):
        App.__init__(self)                  # i/o reset to classes
        self.input  = Input(text)           # use internal string i/o
        self.output = Output() 
        self.input_name  = '<internal>'
        self.output_name = '<internal>'

    def stop(self): 
        App.stop(self)
        return self.output.text             # result = saved output

class RedirectInternalApp(InternalApp):
    def __init__(self, input=''):
        InternalApp.__init__(self, input)          # streams reset to strings
        self.streams = sys.stdin, sys.stdout
        sys.stdin    = self.input                  # for raw_input, stdin
        sys.stdout   = self.output                 # for print, stdout

    def closeApp(self):                            # not __del__
        sys.stdin, sys.stdout = self.streams       # may be redirected


################################################################################
# redirect i/o to classes
# instances of these classes can be used anywhere a file object is expected;
# Note: we could reset/restore streams here instead of in App, but these
# classes are useful outside App, and are independant of App streams;
# there's probably more efficient representations for the file strings;
################################################################################

class FakeStream:
    def close(self):                     # to do: seek(), tell()...
        pass                             # as is: can't back up in file
    def flush(self):
        pass                             # pass: returns None
    def isatty(self):
        return 0

class Input(FakeStream):
    def __init__(self, input):           # input: any sliceable object,
        self.text = input                # but find/getargs want real strings

    def read(self, *size):
        if not size:
            res, self.text = self.text, ''
        else:
            res, self.text = self.text[:size[0]], self.text[size[0]:]
        return res

    def readline(self):
        eoln = string.find(self.text, '\n')
        if eoln == -1: 
            res, self.text = self.text, ''
        else:
            res, self.text = self.text[:eoln+1], self.text[eoln+1:]
        return res

    def readlines(self):
        res = []
        while 1:
            line = self.readline()
            if not line: break
            res.append(line)
        return res

class Output(FakeStream):
    def __init__(self):                            # output in self.output.text
        self.text = ''
    def write(self, string):
        self.text = self.text + string             # to do: handle errors here
    def writelines(self, lines):
        for line in lines: self.write(line)        # or use joinfields

